@@ -13,14 +13,14 @@
 package Messaging::Message::Queue;
 use strict;
 use warnings;
-our $VERSION  = "1.6.1";
-our $REVISION = sprintf("%d.%02d", q$Revision: 1.8 $ =~ /(\d+)\.(\d+)/);
+our $VERSION  = "1.1";
+our $REVISION = sprintf("%d.%02d", q$Revision: 1.4 $ =~ /(\d+)\.(\d+)/);
 
 #
 # used modules
 #
 
-use Messaging::Message qw(_require);
+use Messaging::Message qw(_fatal _require);
 use Params::Validate qw(validate_with :types);
 
 #
@@ -33,9 +33,7 @@ sub new : method {
     $class = shift(@_);
     %option = validate_with(
         params      => \@_,
-        spec        => {
-            type => { type => SCALAR, regex => qr/^[a-zA-Z0-9]+$/ },
-        },
+        spec        => { type => { type => SCALAR, regex => qr/^[a-zA-Z0-9]+$/ } },
         allow_extra => 1,
     );
     $mqc = $class . "::" . $option{type};
@@ -83,11 +81,10 @@ queue (which is the name of the child class), see above
 L<Messaging::Message>,
 L<Messaging::Message::Queue::DQN>,
 L<Messaging::Message::Queue::DQS>,
-L<Messaging::Message::Queue::NULL>,
-L<Messaging::Message::Queue::ZERO>.
+L<Messaging::Message::Queue::NULL>.
 
 =head1 AUTHOR
 
 Lionel Cons L<http://cern.ch/lionel.cons>
 
-Copyright (C) CERN 2011-2016
+Copyright CERN 2011-2012
