@@ -2,7 +2,7 @@
 #                                                                              #
 # File: Messaging/Message/Queue/NULL.pm                                        #
 #                                                                              #
-# Description: abstraction of a null message queue                             #
+# Description: abstraction of a Directory::Queue::Null message queue           #
 #                                                                              #
 #-##############################################################################
 
@@ -13,21 +13,21 @@
 package Messaging::Message::Queue::NULL;
 use strict;
 use warnings;
-our $VERSION  = "1.6.1";
-our $REVISION = sprintf("%d.%02d", q$Revision: 1.7 $ =~ /(\d+)\.(\d+)/);
+our $VERSION  = "1.1";
+our $REVISION = sprintf("%d.%02d", q$Revision: 1.2 $ =~ /(\d+)\.(\d+)/);
 
 #
 # inheritance
 #
 
-our @ISA = qw(Messaging::Message::Queue);
+our @ISA = qw(Messaging::Message::Queue Directory::Queue::Null);
 
 #
 # used modules
 #
 
-use No::Worries::Die qw(dief);
-use Params::Validate qw(validate_with);
+use Messaging::Message qw(_fatal _require);
+use Params::Validate qw(validate_with validate_pos :types);
 
 #
 # constructor
@@ -36,98 +36,29 @@ use Params::Validate qw(validate_with);
 sub new : method {
     my($class, %option, $self);
 
+    _require("Directory::Queue::Null");
     $class = shift(@_);
     %option = validate_with(
         params      => \@_,
         spec        => {},
-        allow_extra => 0,
+	allow_extra => 0,
     );
-    $self = {};
+    $self = Directory::Queue::Null->new(%option);
     bless($self, $class);
     return($self);
 }
 
-#
-# Directory::Queue methods that can be used
-#
-
-sub first : method {
-    return("");
-}
-
-sub next : method { ## no critic 'ProhibitBuiltinHomonyms'
-    return("");
-}
-
-sub count : method {
-    return(0);
-}
-
-sub purge : method {
-}
-
-sub path : method {
-    return("NULL");
-}
-
-sub id : method {
-    return("NULL");
-}
-
-sub add : method {
-    return("");
-}
-
-#
-# Directory::Queue methods that cannot be used
-#
-
-sub touch : method {
-    dief("unsupported method: touch()");
-}
-
-sub lock : method { ## no critic 'ProhibitBuiltinHomonyms'
-    dief("unsupported method: lock()");
-}
-
-sub unlock : method {
-    dief("unsupported method: unlock()");
-}
-
-sub remove : method {
-    dief("unsupported method: remove()");
-}
-
-sub add_ref : method {
-    dief("unsupported method: add_ref()");
-}
-
-sub add_path : method {
-    dief("unsupported method: add_path()");
-}
-
-sub get : method {
-    dief("unsupported method: get()");
-}
-
-sub get_ref : method {
-    dief("unsupported method: get_ref()");
-}
-
-sub get_path : method {
-    dief("unsupported method: get_path()");
-}
-
-sub copy : method {
-    dief("unsupported method: copy()");
-}
-
 #
 # add a message object to the queue
 #
 
 sub add_message : method {
-    return("null");
+    my($self, $msg);
+
+    $self = shift(@_);
+    validate_pos(@_, { isa => "Messaging::Message" });
+    $msg = shift(@_);
+    return($self->add($msg));
 }
 
 #
@@ -135,7 +66,14 @@ sub add_message : method {
 #
 
 sub get_message : method {
-    dief("unsupported method: get_message()");
+    my($self, $elt);
+
+    $self = shift(@_);
+    validate_pos(@_, { type => SCALAR });
+    $elt = shift(@_);
+    # the next line should trigger a fatal error as the queue is always empty
+    $self->get($elt);
+    _fatal("ooops");
 }
 
 1;
@@ -144,7 +82,7 @@ __DATA__
 
 =head1 NAME
 
-Messaging::Message::Queue::NULL - abstraction of a null message queue
+Messaging::Message::Queue::NULL - abstraction of a Directory::Queue::Null message queue
 
 =head1 SYNOPSIS
 
@@ -160,13 +98,15 @@ Messaging::Message::Queue::NULL - abstraction of a null message queue
 
 =head1 DESCRIPTION
 
-This module provides an abstraction of a message queue working a bit like
-C</dev/null>: it can be written to but it always appears empty.
+This module provides an abstraction of a message queue. It derives
+from the L<Directory::Queue::Null> module that provides a generic
+"black hole" queue: added messages will disappear immediately so the
+queue will therefore always appear empty.
 
 =head1 METHODS
 
-In addition to the standard L<Directory::Queue> methods, the following methods
-are available:
+In addition to the methods inherited from L<Directory::Queue::Null>,
+the following methods are available:
 
 =over
 
@@ -187,7 +127,7 @@ get the message from the given element, this generates an error
 
 =head1 SEE ALSO
 
-L<Directory::Queue>,
+L<Directory::Queue::Null>,
 L<Messaging::Message>,
 L<Messaging::Message::Queue>.
 
@@ -195,4 +135,4 @@ L<Messaging::Message::Queue>.
 
 Lionel Cons L<http://cern.ch/lionel.cons>
 
-Copyright (C) CERN 2011-2016
+Copyright CERN 2012
